// Copyright 2000-2021 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.

package org.jetbrains.kotlin.tools.projectWizard.core.service

import kotlin.reflect.KClass
import kotlin.reflect.full.isSubclassOf

class ServicesManager(
    private val services: List<WizardService>,
    private val serviceSelector: (List<WizardService>) -> WizardService?
) {
    @Suppress("UNCHECKED_CAST")
    fun <S : WizardService> serviceByClass(klass: KClass<S>, filter: (S) -> Boolean): S? =
        services.filter { service ->
            service::class.isSubclassOf(klass) && filter(service as S)
        }.let(serviceSelector) as? S

    fun withAdditionalServices(services: List<WizardService>) =
        ServicesManager(this.services + services, serviceSelector)
}