// Copyright 2000-2021 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.

package org.jetbrains.kotlin.idea.asJava

import com.intellij.openapi.project.Project
import com.intellij.psi.*
import org.jetbrains.kotlin.asJava.classes.KtLightClass
import org.jetbrains.kotlin.idea.util.application.getServiceSafe
import org.jetbrains.kotlin.psi.*

interface LightClassProvider {

    fun getLightFieldForCompanionObject(companionObject: KtClassOrObject): PsiField?

    fun getLightClassMethods(function: KtFunction): List<PsiMethod>

    fun getLightClassParameterDeclarations(parameter: KtParameter): List<PsiNamedElement>

    fun getLightClassPropertyDeclarations(property: KtProperty): List<PsiNamedElement>

    fun toLightClassWithBuiltinMapping(classOrObject: KtClassOrObject): PsiClass?

    fun toLightMethods(psiElement: PsiElement): List<PsiMethod>

    fun toLightClass(classOrObject: KtClassOrObject): KtLightClass?

    fun toLightElements(ktElement: KtElement): List<PsiNamedElement>

    fun createKtFakeLightClass(kotlinOrigin: KtClassOrObject): PsiClass?

    fun getRepresentativeLightMethod(psiElement: PsiElement): PsiMethod?

    fun isKtFakeLightClass(psiClass: PsiClass): Boolean

    fun isKtLightClassForDecompiledDeclaration(psiClass: PsiClass): Boolean

    fun createKtFakeLightMethod(ktDeclaration: KtNamedDeclaration): PsiMethod?

    companion object {

        fun getInstance(project: Project): LightClassProvider = project.getServiceSafe()

        fun providedGetLightFieldForCompanionObject(companionObject: KtClassOrObject): PsiField? =
            getInstance(companionObject.project).getLightFieldForCompanionObject(companionObject)

        fun providedGetLightClassMethods(function: KtFunction): List<PsiMethod> =
            getInstance(function.project).getLightClassMethods(function)

        fun providedGetLightClassParameterDeclarations(parameter: KtParameter): List<PsiNamedElement> =
            getInstance(parameter.project).getLightClassParameterDeclarations(parameter)

        fun providedGetLightClassPropertyDeclarations(property: KtProperty): List<PsiNamedElement> =
            getInstance(property.project).getLightClassPropertyDeclarations(property)

        fun KtClassOrObject.providedToLightClassWithBuiltinMapping(): PsiClass? =
            getInstance(project).toLightClassWithBuiltinMapping(this)

        fun PsiElement.providedToLightMethods(): List<PsiMethod> =
            getInstance(project).toLightMethods(this)

        fun KtClassOrObject.providedToLightClass(): KtLightClass? =
            getInstance(project).toLightClass(this)

        fun KtElement.providedToLightElements(): List<PsiNamedElement> =
            getInstance(project).toLightElements(this)

        fun providedCreateKtFakeLightClass(kotlinOrigin: KtClassOrObject): PsiClass? =
            getInstance(kotlinOrigin.project).createKtFakeLightClass(kotlinOrigin)

        fun PsiClass.providedIsKtFakeLightClass(): Boolean =
            getInstance(project).isKtFakeLightClass(this)

        fun PsiClass.providedIsKtLightClassForDecompiledDeclaration(): Boolean =
            getInstance(project).isKtLightClassForDecompiledDeclaration(this)

        fun providedCreateKtFakeLightMethod(ktDeclaration: KtNamedDeclaration): PsiMethod? =
            getInstance(ktDeclaration.project).createKtFakeLightMethod(ktDeclaration)

        fun PsiElement.providedGetRepresentativeLightMethod(): PsiMethod? =
            getInstance(project).getRepresentativeLightMethod(this)
    }
}